@@ -12,11 +12,12 @@ source "$(dirname "$0")/setup.sh"
 # Clone NSPR.
 hg_clone https://hg.mozilla.org/projects/nspr nspr default
 
-if [[ -f nss/nspr.patch && "$ALLOW_NSPR_PATCH" == "1" ]]; then
-  pushd nspr
+pushd nspr
+hg revert --all
+if [[ -f ../nss/nspr.patch && "$ALLOW_NSPR_PATCH" == "1" ]]; then
   cat ../nss/nspr.patch | patch -p1
-  popd
 fi
+popd
 
 # Build.
 make -C nss nss_build_all
