@@ -450,7 +450,7 @@ SECStatus readPSK(const char *arg, SECItem *psk, SECItem *label);
 
 void printflags(char *trusts, unsigned int flags);
 
-#if !defined(XP_UNIX) && !defined(XP_OS2)
+#if !defined(XP_UNIX)
 extern int ffs(unsigned int i);
 #endif
 
