@@ -3,14 +3,17 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include <cassert>
-#include <cstdint>
 #include <cstddef>
+#include <cstdint>
+#include <iostream>
 #include <memory>
 
 #include "blapi.h"
 #include "shared.h"
 #include "ssl.h"
+#include "sslexp.h"
 #include "sslimpl.h"
+#include "sslt.h"
 #include "tls_client_config.h"
 #include "tls_common.h"
 #include "tls_mutators.h"
@@ -20,90 +23,35 @@
 __attribute__((constructor)) static void set_is_dtls() {
   TlsMutators::SetIsDTLS();
 }
-#endif
 
-const PRUint8 kDummyPskIdentity[] = "fuzz-identity";
-
-#ifndef IS_DTLS_FUZZ
-static const HpkeSymmetricSuite kDummyEchHpkeSuites[] = {
-    {HpkeKdfHkdfSha256, HpkeAeadChaCha20Poly1305},
-    {HpkeKdfHkdfSha256, HpkeAeadAes128Gcm}};
-
-static PRUint8 DummyEchConfigs[1000];
-static unsigned int DummyEchConfigsLen;
-
-static SECStatus InitializeDummyEchConfigs() {
-  const sslNamedGroupDef* group_def =
-      ssl_LookupNamedGroup(ssl_grp_ec_curve25519);
-  assert(group_def);
-
-  const SECOidData* oid_data = SECOID_FindOIDByTag(group_def->oidTag);
-  assert(oid_data);
-
-  ScopedSECItem params(
-      SECITEM_AllocItem(nullptr, nullptr, (2 + oid_data->oid.len)));
-  assert(params);
-
-  params->data[0] = SEC_ASN1_OBJECT_ID;
-  params->data[1] = oid_data->oid.len;
-  PORT_Memcpy((void*)(params->data + 2), oid_data->oid.data, oid_data->oid.len);
-
-  SECKEYPublicKey* pub_key = nullptr;
-  SECKEYPrivateKey* priv_key =
-      SECKEY_CreateECPrivateKey(params.get(), &pub_key, nullptr);
-  assert(pub_key);
-  assert(priv_key);
-
-  SECStatus rv = SSL_EncodeEchConfigId(
-      77, "fuzz.name", 100, HpkeDhKemX25519Sha256, pub_key, kDummyEchHpkeSuites,
-      sizeof(kDummyEchHpkeSuites) / sizeof(HpkeSymmetricSuite), DummyEchConfigs,
-      &DummyEchConfigsLen, sizeof(DummyEchConfigs));
-
-  SECKEY_DestroyPublicKey(pub_key);
-  SECKEY_DestroyPrivateKey(priv_key);
-
-  return rv;
-}
-#endif  // IS_DTLS_FUZZ
-
-PRFileDesc* ImportFD(PRFileDesc* model, PRFileDesc* fd) {
-#ifdef IS_DTLS_FUZZ
-  return DTLS_ImportFD(model, fd);
+#define ImportFD DTLS_ImportFD
 #else
-  return SSL_ImportFD(model, fd);
-#endif
-}
+#define ImportFD SSL_ImportFD
 
-static SECStatus DummyCompressionEncode(const SECItem* input, SECItem* output) {
-  SECITEM_CopyItem(NULL, output, input);
-  PORT_Memcpy(output->data, input->data, output->len);
+static PRUint8 gEchConfigs[1024];
+static unsigned int gEchConfigsLen;
 
-  return SECSuccess;
-}
+const HpkeSymmetricSuite kEchHpkeCipherSuites[] = {
+    {HpkeKdfHkdfSha256, HpkeAeadAes128Gcm},
+    {HpkeKdfHkdfSha256, HpkeAeadAes256Gcm},
+    {HpkeKdfHkdfSha256, HpkeAeadChaCha20Poly1305},
 
-static SECStatus DummyCompressionDecode(const SECItem* input,
-                                        unsigned char* output, size_t outputLen,
-                                        size_t* usedLen) {
-  assert(input->len == outputLen);
-  PORT_Memcpy(output, input->data, input->len);
-  *usedLen = outputLen;
+    {HpkeKdfHkdfSha384, HpkeAeadAes128Gcm},
+    {HpkeKdfHkdfSha384, HpkeAeadAes256Gcm},
+    {HpkeKdfHkdfSha384, HpkeAeadChaCha20Poly1305},
 
-  return SECSuccess;
-}
-
-static SECStatus AuthCertificateHook(void* arg, PRFileDesc* fd, PRBool checksig,
-                                     PRBool isServer) {
-  assert(!isServer);
-
-  auto config = reinterpret_cast<ClientConfig*>(arg);
-  if (config->FailCertificateAuthentication()) return SECFailure;
-
-  return SECSuccess;
-}
+    {HpkeKdfHkdfSha512, HpkeAeadAes128Gcm},
+    {HpkeKdfHkdfSha512, HpkeAeadAes256Gcm},
+    {HpkeKdfHkdfSha512, HpkeAeadChaCha20Poly1305},
+};
+#endif  // IS_DTLS_FUZZ
+const SSLCertificateCompressionAlgorithm kCompressionAlg = {
+    0x1337, "fuzz", DummyCompressionEncode, DummyCompressionDecode};
+const PRUint8 kPskIdentity[] = "fuzz-identity";
 
 static void SetSocketOptions(PRFileDesc* fd,
                              std::unique_ptr<ClientConfig>& config) {
-  SECStatus rv = SSL_OptionSet(fd, SSL_NO_CACHE, config->EnableCache());
+  SECStatus rv = SSL_OptionSet(fd, SSL_NO_CACHE, config->NoCache());
   assert(rv == SECSuccess);
 
   rv = SSL_OptionSet(fd, SSL_ENABLE_EXTENDED_MASTER_SECRET,
@@ -134,33 +82,11 @@ static void SetSocketOptions(PRFileDesc* fd,
                      config->EnableCHExtensionPermutation());
   assert(rv == SECSuccess);
 
-#ifndef IS_DTLS_FUZZ
-  rv =
-      SSL_OptionSet(fd, SSL_ENABLE_RENEGOTIATION, SSL_RENEGOTIATE_UNRESTRICTED);
-  assert(rv == SECSuccess);
-#endif
-
   if (config->SetCertificateCompressionAlgorithm()) {
-    rv = SSLExp_SetCertificateCompressionAlgorithm(
-        fd, {
-                .id = 0x1337,
-                .name = "fuzz-compression",
-                .encode = DummyCompressionEncode,
-                .decode = DummyCompressionDecode,
-            });
+    rv = SSL_SetCertificateCompressionAlgorithm(fd, kCompressionAlg);
     assert(rv == SECSuccess);
   }
 
-#ifndef IS_DTLS_FUZZ
-  if (config->SetClientEchConfigs()) {
-    rv = InitializeDummyEchConfigs();
-    assert(rv == SECSuccess);
-
-    rv = SSL_SetClientEchConfigs(fd, DummyEchConfigs, DummyEchConfigsLen);
-    assert(rv == SECSuccess);
-  }
-#endif
-
   if (config->SetVersionRange()) {
     rv = SSL_VersionRangeSet(fd, &config->VersionRange());
     assert(rv == SECSuccess);
@@ -174,8 +100,8 @@ static void SetSocketOptions(PRFileDesc* fd,
         PK11_KeyGen(slot, CKM_NSS_CHACHA20_POLY1305, nullptr, 32, nullptr);
     assert(key);
 
-    rv = SSL_AddExternalPsk(fd, key, kDummyPskIdentity, sizeof(kDummyPskIdentity) - 1,
-                            ssl_hash_sha256);
+    rv = SSL_AddExternalPsk(fd, key, kPskIdentity, sizeof(kPskIdentity) - 1,
+                            config->PskHashType());
     assert(rv == SECSuccess);
 
     PK11_FreeSlot(slot);
@@ -185,6 +111,75 @@ static void SetSocketOptions(PRFileDesc* fd,
   rv = SSL_OptionSet(fd, SSL_ENABLE_POST_HANDSHAKE_AUTH,
                      config->EnablePostHandshakeAuth());
   assert(rv == SECSuccess);
+
+  rv = SSL_OptionSet(fd, SSL_ENABLE_0RTT_DATA, config->EnableZeroRtt());
+  assert(rv == SECSuccess);
+
+  rv = SSL_OptionSet(fd, SSL_ENABLE_ALPN, config->EnableAlpn());
+  assert(rv == SECSuccess);
+
+  rv =
+      SSL_OptionSet(fd, SSL_ENABLE_FALLBACK_SCSV, config->EnableFallbackScsv());
+  assert(rv == SECSuccess);
+
+  rv =
+      SSL_OptionSet(fd, SSL_ENABLE_OCSP_STAPLING, config->EnableOcspStapling());
+  assert(rv == SECSuccess);
+
+  rv = SSL_OptionSet(fd, SSL_ENABLE_SESSION_TICKETS,
+                     config->EnableSessionTickets());
+  assert(rv == SECSuccess);
+
+  rv = SSL_OptionSet(fd, SSL_ENABLE_TLS13_COMPAT_MODE,
+                     config->EnableTls13CompatMode());
+  assert(rv == SECSuccess);
+
+  rv = SSL_OptionSet(fd, SSL_NO_LOCKS, config->NoLocks());
+  assert(rv == SECSuccess);
+
+#ifndef IS_DTLS_FUZZ
+  rv =
+      SSL_OptionSet(fd, SSL_ENABLE_RENEGOTIATION, SSL_RENEGOTIATE_UNRESTRICTED);
+  assert(rv == SECSuccess);
+
+  if (config->SetClientEchConfigs()) {
+    const sslNamedGroupDef* group_def =
+        ssl_LookupNamedGroup(ssl_grp_ec_curve25519);
+    assert(group_def);
+
+    const SECOidData* oid_data = SECOID_FindOIDByTag(group_def->oidTag);
+    assert(oid_data);
+
+    ScopedSECItem params(
+        SECITEM_AllocItem(nullptr, nullptr, (2 + oid_data->oid.len)));
+    assert(params);
+
+    params->data[0] = SEC_ASN1_OBJECT_ID;
+    params->data[1] = oid_data->oid.len;
+    PORT_Memcpy((void*)(params->data + 2), oid_data->oid.data,
+                oid_data->oid.len);
+
+    SECKEYPublicKey* pub_key = nullptr;
+    SECKEYPrivateKey* priv_key =
+        SECKEY_CreateECPrivateKey(params.get(), &pub_key, nullptr);
+    assert(pub_key);
+    assert(priv_key);
+
+    rv = SSL_EncodeEchConfigId(
+        77, "fuzz.name", 100, HpkeDhKemX25519Sha256, pub_key,
+        kEchHpkeCipherSuites,
+        sizeof(kEchHpkeCipherSuites) / sizeof(HpkeSymmetricSuite), gEchConfigs,
+        &gEchConfigsLen, sizeof(gEchConfigs));
+
+    SECKEY_DestroyPublicKey(pub_key);
+    SECKEY_DestroyPrivateKey(priv_key);
+
+    assert(rv == SECSuccess);
+
+    rv = SSL_SetClientEchConfigs(fd, gEchConfigs, gEchConfigsLen);
+    assert(rv == SECSuccess);
+  }
+#endif  // IS_DTLS_FUZZ
 }
 
 // This is only called when we set SSL_ENABLE_FALSE_START=1,
@@ -195,8 +190,19 @@ static SECStatus CanFalseStartCallback(PRFileDesc* fd, void* arg,
   return SECSuccess;
 }
 
-static void SetupCallbacks(PRFileDesc* fd, ClientConfig* config) {
-  SECStatus rv = SSL_AuthCertificateHook(fd, AuthCertificateHook, config);
+static SECStatus AuthCertificateHook(void* arg, PRFileDesc* fd, PRBool checksig,
+                                     PRBool isServer) {
+  assert(!isServer);
+
+  auto config = reinterpret_cast<ClientConfig*>(arg);
+  if (config->FailCertificateAuthentication()) return SECFailure;
+
+  return SECSuccess;
+}
+
+static void SetupCallbacks(PRFileDesc* fd,
+                           std::unique_ptr<ClientConfig>& config) {
+  SECStatus rv = SSL_AuthCertificateHook(fd, AuthCertificateHook, config.get());
   assert(rv == SECSuccess);
 
   rv = SSL_SetCanFalseStartCallback(fd, CanFalseStartCallback, nullptr);
@@ -204,8 +210,7 @@ static void SetupCallbacks(PRFileDesc* fd, ClientConfig* config) {
 }
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t len) {
-  std::unique_ptr<NSSDatabase> db(new NSSDatabase());
-  assert(db != nullptr);
+  static std::unique_ptr<NSSDatabase> db(new NSSDatabase());
 
   EnableAllProtocolVersions();
   std::unique_ptr<ClientConfig> config(new ClientConfig(data, len));
@@ -220,13 +225,18 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t len) {
   PRFileDesc* ssl_fd = ImportFD(nullptr, fd.get());
   assert(ssl_fd == fd.get());
 
+  // Print the client config for debugging.
+  if (ssl_trace >= 90) {
+    std::cerr << config << "\n";
+  }
+
   // Probably not too important for clients.
   SSL_SetURL(ssl_fd, "server");
 
   FixTime(ssl_fd);
   SetSocketOptions(ssl_fd, config);
   EnableAllCipherSuites(ssl_fd);
-  SetupCallbacks(ssl_fd, config.get());
+  SetupCallbacks(ssl_fd, config);
   DoHandshake(ssl_fd, false);
 
   // Release all SIDs.
