@@ -20,19 +20,15 @@ struct CTSContextStr {
 
 CTSContext *
 CTS_CreateContext(void *context, freeblCipherFunc cipher,
-                  const unsigned char *iv, unsigned int blocksize)
+                  const unsigned char *iv)
 {
     CTSContext *cts;
 
-    if (blocksize > MAX_BLOCK_SIZE) {
-        PORT_SetError(SEC_ERROR_LIBRARY_FAILURE);
-        return NULL;
-    }
     cts = PORT_ZNew(CTSContext);
     if (cts == NULL) {
         return NULL;
     }
-    PORT_Memcpy(cts->iv, iv, blocksize);
+    PORT_Memcpy(cts->iv, iv, MAX_BLOCK_SIZE);
     cts->cipher = cipher;
     cts->context = context;
     return cts;
@@ -217,12 +213,12 @@ CTS_DecryptUpdate(CTSContext *cts, unsigned char *outbuf,
         if (inbuf != outbuf) {
             memcpy(outbuf, inbuf, inlen);
             /* keep the names so we logically know how we are using the
-         * buffers */
+             * buffers */
             inbuf = outbuf;
         }
         memcpy(lastBlock, inbuf + inlen - blocksize, blocksize);
         /* we know inbuf == outbuf now, inbuf is declared const and can't
-     * be the target, so use outbuf for the target here */
+         * be the target, so use outbuf for the target here */
         memcpy(outbuf + inlen - pad, inbuf + inlen - blocksize - pad, pad);
         memcpy(outbuf + inlen - blocksize - pad, lastBlock, blocksize);
     }
